#include <stdio.h>
#include <ctype.h>
#include <GL/glut.h>

void display(void);
void draw_teapot(void);
void draw_cube(void);
void reshape(int x, int y);
void keyb(unsigned char key, int x, int y);

struct object {
	float x, y, z;	/* position */
	float size;		/* cube size */
} cubes[] = {
	{-4, 0, 0, 0.5},
	{4, 0, 0, 0.5},
	{0, 0, -4, 0.5},
	{0, 0, 4, 0.5}
};

/* number of cubes defined above */
int num_cubes = sizeof cubes / sizeof cubes[0];


float angle;

int main(int argc, char **argv)
{
	/* initialize glut, and create a window */
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(800, 600);
	glutCreateWindow("OpenGL window");

	/* register event callback handlers */
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);

	/* enter the glut main event handling loop, this never returns */
	glutMainLoop();
	return 0;
}

/* the display callback is called when we need to redraw the graphics in our
 * OpenGL window (due to X11 Expose event, or call to glutPostRedisplay).
 */
void display(void)
{
	float lpos[] = {4, 5, 10, 1};
	int i;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);

	/* set view matrix by calling gluLookAt */
	glLoadIdentity();
	gluLookAt(-3, 6, 8, 0, 0, 0, 0, 1, 0);
	glRotatef(angle, 0, 1, 0);

	glLightfv(GL_LIGHT0, GL_POSITION, lpos);

	for(i=0; i<num_cubes; i++) {
		/* save current matrix */
		glPushMatrix();

		/* multiply object's world matrix by calling
			glTranslate/glRotate helper functions */
		glTranslatef(cubes[i].x, cubes[i].y, cubes[i].z);
		glScalef(cubes[i].size, cubes[i].size, cubes[i].size);

		draw_cube(); /* draw the cube */

		/* restore the view matrix by popping */
		glPopMatrix();
	}

	draw_teapot();

	glutSwapBuffers();

}

void draw_teapot(void)
{
	float color[] = {0.1, 0.3, 1, 1};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, color);
	glutSolidTeapot(1.5);
}

void draw_cube(void)
{
	float color[] = {1, 0, 0, 1};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, color);

	glBegin(GL_QUADS);
	/* far face (-Z) */
	glNormal3f(0, 0, -1);
	glColor3f(0, 0, 1);
	glVertex3f(1, -1, -1);
	glVertex3f(-1, -1, -1);
	glVertex3f(-1, 1, -1);
	glVertex3f(1, 1, -1);
	/* top face (+Y) */
	glNormal3f(0, 1, 0);
	glColor3f(0, 1, 1);
	glVertex3f(-1, 1, 1);
	glVertex3f(1, 1, 1);
	glVertex3f(1, 1, -1);
	glVertex3f(-1, 1, -1);
	/* bottom face (-Y) */
	glNormal3f(0, -1, 0);
	glColor3f(1, 0, 1);
	glVertex3f(-1, -1, -1);
	glVertex3f(1, -1, -1);
	glVertex3f(1, -1, 1);
	glVertex3f(-1, -1, 1);
	/* right face (+X) */
	glNormal3f(1, 0, 0);
	glColor3f(0, 1, 0);
	glVertex3f(1, -1, 1);
	glVertex3f(1, -1, -1);
	glVertex3f(1, 1, -1);
	glVertex3f(1, 1, 1);
	/* left face (-X) */
	glNormal3f(-1, 0, 0);
	glColor3f(1, 1, 0);
	glVertex3f(-1, -1, -1);
	glVertex3f(-1, -1, 1);
	glVertex3f(-1, 1, 1);
	glVertex3f(-1, 1, -1);
	/* near face (+Z) */
	glNormal3f(0, 0, 1);
	glColor3f(1, 0, 0);
	glVertex3f(-1, -1, 1);
	glVertex3f(1, -1, 1);
	glVertex3f(1, 1, 1);
	glVertex3f(-1, 1, 1);
	glEnd();
}

/* the reshape callback is called when our window is resized
 * (X11 ConfigureNotify event)
 */
void reshape(int x, int y)
{
	glViewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)x / (float)y, 1.0, 1000.0);
}

/* the keyboard callback is called when a key is pressed
 * (X11 KeyPress event)
 */
void keyb(unsigned char key, int x, int y)
{
	switch(key) {
	case ',':
		angle -= 2;
		glutPostRedisplay();
		break;

	case '.':
		angle += 2;
		glutPostRedisplay();
		break;
	}
}
